;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x96fd364f4d724f295eab68cfc53798dc . "./code.cg2js/cg-common-control.fasl")
(#x8e0e9050ded4a32c33e91ab712553913 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xcb41995931ec87611d877256d81250b9 . "./code.cg2js/cg-yes-no-string.fasl")
(#x52a1fa8f83d881e5984a3d096b48487c . "./code.cg2js/cg-clipboard.fasl")
(#x0440659fe5046c27a379bfc227e7a901 . "./code.cg2js/cg-dde-utils.fasl")
(#x974950f423f7377e582315782ce35493 . "./code.cg2js/cg-editable-text.fasl")
(#xd646f02d782dd96c8a12280c22bfe933 . "./code.cg2js/cg-text-or-combo.fasl")
(#x21b87562a5d1714faee94e4b7eda81ca . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x124343a94299d9e05483cdfa36c82875 . "./code.cg2js/cg-graphics-context.fasl")
(#x97eab6e72bb1e29eaca6a949e991131b . "./code.cg2js/cg-choose-printer.fasl")
(#xb6e91f95943fdd45b8f1f34fdb78dddc . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x4fc33490ac5d5596870c47ad91847e44 . "./code.cg2js/cg-edit-in-place.fasl")
(#xf7d175482269a7e60f3cfcce087c7d36 . "./code.cg2js/cg-split-bar.fasl")
(#x4e80b0b05e85b8cc0870dc03d5572200 . "./code.cg2js/cg-timer.fasl")
(#x7b63a0e2505b56da799adb23d5da8fa1 . "./code.cg2js/cg-progress-indicator.fasl")
(#x7b354557f1f435be2224f1e396f3f95c . "./code.cg2js/cg-object-editor.fasl")
(#xbca88661eac1d50a41a9e5475525b698 . "./code.cg2js/cg-fill-texture.fasl")
(#xc8ffb1e4006ffd737adfe61d067516d5 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xd298ed753c8feecebc6c135f71efe538 . "./code.cg2js/cg-os-window.fasl")
(#xa1268587a8fc7ec73239acc41c0db49e . "./code.cg2js/cg-color-dialog.fasl")
(#x3fa9547681acc4a6b33334ae17ee52de . "./code.cg2js/cg-combo-box.fasl")
(#x6faee2cdab203a71a02e5cae64195dd3 . "./code.cg2js/cg-directory-dialog.fasl")
(#xa15e99c6321d0bff6ef3b68269a45966 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x219f22c9f4284af6a85be48656974f3b . "./code.cg2js/cg-tooltip.fasl")
(#xe9ea9323fa78831d97f2493ed16d5875 . "./code.cg2js/cg-lettered-menu.fasl")
(#x4b2d496383bbf55b1a7b396d887086e2 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x85da56da84c21275a471ea3b2bd15b64 . "./code.cg2js/cg-pixmap.fasl")
(#x797b19573d94730523d4e00f3025a3a3 . "./code.cg2js/cg.fasl")
(#x2963e92d4078e054f6832a6dc0a7ad91 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xf5c1a00c181da6cadeff14836b95027a . "./code.cg2js/cg-template-string.fasl")
(#x826d25f0d0c16544bc9c29cd643b4ca1 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xc3c94522f6fc809307d69d8011f0062b . "./code.cg2js/cg-check-box.fasl")
(#x8470ec6a8c5848e29302f16645a383e6 . "./code.cg2js/cg-toggling-widget.fasl")
(#xce1e2e41b3225f6b2eadca282f4914ce . "./code.cg2js/cg-radio-button.fasl")
(#x1ca15245df360a2468216508eb2f8121 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x33f9a706b40180f73b756bc0b7fce699 . "./code.cg2js/cg-scaling-stream.fasl")
(#xb5bd551ef244e90b7c4c39514df4a496 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x478d4b4ba6334f4f96d32f9cc9a55417 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xbf466886b8a4ae0a92c3b519208b2460 . "./code.cg2js/cg-grid-widget.fasl")
(#x2c517877515c785e9986e752a1bcdaaf . "./code.cg2js/cg-text-edit-pane.fasl")
(#xf746acef00ceada909f8450fc7380fe4 . "./code.cg2js/cg-item-list.fasl")
(#x90b7a2e96d56999cca1b439e61907b38 . "./code.cg2js/cg-paren-matching.fasl")
(#x480229274e942e7fec8c2d40efb7a3eb . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xd0d042100f50493f1f98b68db22b89fe . "./code.cg2js/cg-html-dialog.fasl")
(#x803ef058ca1ce977bc6dac8695a51a8b . "./code.cg2js/cg-pixmap-widget.fasl")
(#x722d3e4654f1945fa1e62139825e00a5 . "./code.cg2js/cg-project-window.fasl")
(#xe9ea7c76d6059ad0efa285ad44f0d9a3 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xd25d54d707d4825b2c5bf8e50de87c3b . "./code.cg2js/cg-toolbar.fasl")
(#x951cb8a0fbbba35bc21d681f2fa9e89b . "./code.cg2js/cg-caret.fasl")
(#x8990a8c61dbcb891013c0286281eca69 . "./code.cg2js/cg-scroll-bar.fasl")
(#x1c499441cea5a83fd433d8c999d9037c . "./code.cg2js/cg-web-browser.fasl")
(#x66d5517e3ded252b6211dd034242c86d . "./code.cg2js/cg-dropping-outline.fasl")
(#x0fe00293d6b21981b9ef6602790a7ada . "./code.cg2js/cg-font-dialog.fasl")
(#x693dd3c3dfd087f607d33ce99be7588d . "./code.cg2js/cg-icon-pixmap.fasl")
(#x9e8a7cb8b4502f06f2321ebd050de404 . "./code.cg2js/cg-yes-no-list.fasl")
(#xa8c459bdb9a8e1a434308a0a6d271278 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x8b29ceccc39f31dc545d976e96b1d709 . "./code.cg2js/cg-lamp.fasl")
(#x3d7e7e2ae4354e1054ae31fe47771937 . "./code.cg2js/cg-drawable.fasl")
(#x1085d334b3df573b23b720a2f13efd72 . "./code.cg2js/cg-choice-list.fasl")
(#x501d9746b0506ed874e35ddbd0eef9e3 . "./code.cg2js/cg-curve.fasl")
(#x1c6607d6f7a385b10b47a74549d5d711 . "./code.cg2js/cg-base.fasl")
(#x67ea0529c7b7da007edaea990e73b039 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x0a0196348067c9300223083bdeab83e1 . "./code.cg2js/cg-printing.fasl")
(#x1df8553128e1d5ab70c3541a816757e2 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xaab871a3e11b062b7526ab76b0c1e482 . "./code.cg2js/cg-hotspot.fasl")
(#x626fb02822967631a284c21fb8dd4ccf . "./code.cg2js/cg-object-editor-layout.fasl")
(#x94e3aa5c8da9a4ffd32b927f0bd1dd15 . "./code.cg2js/cg-utility-dialog.fasl")
(#xb72278362c54c226a95299f89f7c46f7 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x1a97acbc57f2fca9f99c38586fc3846f . "./code.cg2js/cg-comtab.fasl")
(#x09898ef840a6a9571758a703b181cb13 . "./code.cg2js/cg-os-widget.fasl")
(#xbe5846f6c960e78ed573b5810e255fc9 . "./code.cg2js/cg-icon.fasl")
(#x96c1f65260ee3b8bc114df5d6260fb16 . "./code.cg2js/cg-dev.fasl")
(#xb123181dd383469fd911305f7d64ba40 . "./code.cg2js/cg-calendar.fasl")
(#x2085b1616fe6c3a11197de566704ebaf . "./code.cg2js/cg-group-box.fasl")
(#x63645592f77a54c57e3867019d674c4f . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x8df400089ac3eca789cca5bf0e1a0795 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xf888024437bdee31a837a12666c0a276 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x863664c3ee750eea2928505fd038169f . "./code.cg2js/cg-nodes.fasl")
(#xff8c925de217bad8fd27c9ced2da547c . "./code.cg2js/cg-intersect.fasl")
(#x6fe0c1654b50c63c90262758935b284b . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xc67bc7142690dc6f76bc76b6b3b35a36 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x62623fc9b3f2ac2f31d8d5c0522b8c5e . "./code.cg2js/cg-class-support.fasl")
(#xccc8c39bea990777cf863b6e5ce0266c . "./code.cg2js/cg-dialog-item.fasl")
(#x8ac5c901d06936014df35255ca64a34c . "./code.cg2js/cg-plot-widget.fasl")
(#xd214ad4b9810b489d2a71d6fc0e7d0b0 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x4a696aa1fbfca94fb95075cd16ee3092 . "./code.cg2js/cg-up-down-control.fasl")
(#x6a0af51731652deffc098705e691775a . "./code.cg2js/cg-rich-edit.fasl")
(#x889e115c3c46df3b4a607d0ed637ec8e . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x881cee12a545df8ad9a4025cf822833c . "./code.cg2js/cg-status-bar.fasl")
(#x180b529e27f2f4745868fddd860056fd . "./code.cg2js/cg-class-grid.fasl")
(#x480b7aee2044b2f9771dadf52ce085cb . "./code.cg2js/cg-wrap-string.fasl")
(#x4a6a35d1fcd4e139695b1dc9ac4eb0e7 . "./code.cg2js/cg-layout.fasl")
(#x430449e787e8dd4a82c2814953a42e8c . "./code.cg2js/cg-lisp-widget.fasl")
(#xfb79d31d9da3a73c1accbeff52a4f30c . "./code.cg2js/cg-button.fasl")
(#xe9ffbe1ac02cba16e9ce51651eeaec10 . "./code.cg2js/cg-string-dialog.fasl")
(#x9156036c03372008b7d25e00e3c668e3 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x610683f2fb166b40852e6752b5b43743 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x2bcb9b5e2580cea943c56d01af6a6d71 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xe2ad10818deb1729850d6cb9b4bd4589 . "./code.cg2js/cg-static-text.fasl")
(#x089a56480b21003649d9181faa918b2c . "./code.cg2js/cg-outline.fasl")
(#xfe982972f175db971cdfb3e0121d9cbe . "./code.cg2js/cg-gesture-emulation.fasl")
(#x55c4292d7a63c6da5e027c722b81460f . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x7b64b6a15c67d330e64917a7c3f6f908 . "./code.cg2js/cg-picture-widget.fasl")
(#xab237479deffa235cea6c49543f001d9 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x41851e53540494e90bff76bc5f144736 . "./code.cg2js/cg-chart-widget.fasl")
(#xd22d051dd0e92f0335eee9711338869b . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xcd32a26002e6d2ed81bf93d546f957f0 . "./code.cg2js/cg-lisp-text.fasl")
(#x3c5010532bae2c4ddf4c999cd1cb4458 . "./code.cg2js/cg-palette.fasl")
(#x5de9eadb69b2de18d0c5902b87a306ec . "./code.cg2js/cg-property.fasl")
(#x82392ab7c52c34aeab4f155b8bdfc21a . "./code.cg2js/cg-intersect-posbox.fasl")
(#x5989e241e2ea1f4bba2828caccb2a8d9 . "./code.cg2js/cg-file-dialog.fasl")
(#x756dfdbe826828e8a1583b5d4ab98dba . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x34d32db9435377a45dfe31e6687bf9b9 . "./code.cg2js/cg-get-pixmap.fasl")
(#x7776bb0001da56991f05fc49352c0a51 . "./code.cg2js/cg-tray.fasl")
(#xb38d0cf651e8eff38651a6fa2bb6269e . "./code.cg2js/cg-acache.fasl")
(#xf878ccf2f8df6be911bc520b81c2cd63 . "./code.cg2js/cg-message-dialog.fasl")
(#x42466366eac2d1c97c8e4ee804420bd3 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xd40d98c47f7755936c7636b65c116d10 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x549813071b5491a1a0189861fc497a57 . "./code.cg2js/cg-tab-control.fasl")
(#x06f74e122a18c7c2b12fe52013a53444 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xaac33d69541bface80edfa06d008649a . "./code.cg2js/cg-menu-tooltip.fasl")
(#xe2bb96f4009a592f3cea29ecdfa022b4 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xe19d22730c2d7fb67dd36f40d2f8f6ba . "./code.cg2js/cg-menu.fasl")
(#xeec3ffd55ab6bec54bd6659078ef4f12 . "./code.cg2js/cg-get-position.fasl")
(#x3de152fee9471a4edff78fada50a73bd . "./code.cg2js/cg-trackbar.fasl")
(#x59f78f9cb2e81dd61124df0f80ec0516 . "./code.cg2js/cg-selected-object.fasl")
(#x8329147d45865c29a202fafea419382c . "./code.cg2js/cg-list-view.fasl")
(#xdc794843fc6b6a3ece0d07a7b9bd4363 . "./code.cg2js/cg-html-widget.fasl")
(#x80b5ad412ada71c5cca0d377d7558865 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x86d3afe90f05cab54a9d31df5b2e1a7f . "./code.cg2js/cg-text-widget.fasl")
(#x9dd4917bc79559ccb873b10b90713990 . "./code.idejs/ide-base.fasl")
(#x1779cedc0c1d666eef2953053b4f2042 . "./code.idejs/ide-process-browser.fasl")
(#xfd4cfc657ab44ce2ca8f6ee66c440e9e . "./code.idejs/ide-patch-dialog.fasl")
(#xb6281435f76d7c9280f4ea90bda2bbf1 . "./code.idejs/ide-keyshort.fasl")
(#xaad3fd104595241972fbfdc2876d8d3e . "./code.idejs/ide-window-list.fasl")
(#xbb1faebd87ea73bbcd41487c39c6ecca . "./code.idejs/ide.fasl")
(#xf09ad9dd73d4b7f9afa0a53e2013f50b . "./code.idejs/ide-editor.fasl")
(#x7cf22e9bd564e55f4b6eb60a5113935a . "./code.idejs/ide-clipboard.fasl")
(#x69cb094c9783c38fa52bab239d645fa7 . "./code.idejs/ide-trace.fasl")
(#x038c4d2a2b62c69b63e589465dfc4290 . "./code.idejs/ide-class-browser.fasl")
(#x2d3684a1b69b56300f4fb449f2de837e . "./code.idejs/ide-grapher.fasl")
(#x572d5fba9ccc1c6e42c1bc2dbbf69447 . "./code.idejs/ide-apropos.fasl")
(#x60d95616b61ed1f1d18e9cf16b3477c6 . "./code.idejs/ide-navigator.fasl")
(#xb28db6db14879cb879bb9a0fae79ee6c . "./code.idejs/ide-find-definitions.fasl")
(#x1d6f256d92ad6c074dca76f9fb6cae60 . "./code.idejs/ide-project.fasl")
(#x8c2f333b2a68290cb13e69afde809d87 . "./code.idejs/ide-class-grapher.fasl")
(#x07efea4316ebdd3c0a4175506899a080 . "./code.idejs/ide-acache.fasl")
(#x6addd41e0350fe5e9d776b81524792f9 . "./code.idejs/ide-find-in-files.fasl")
(#xbc5873da2830c06455191b2c67892121 . "./code.idejs/ide-inspector.fasl")
(#xf4ee6e6e652b04a00821807d8aa5f55f . "./code.idejs/ide-intro.fasl")
(#x7e35c6fe677c6c959a5b5d0c9990873f . "./code.idejs/ide-package-list.fasl")
(#x53b60c71c5b8ee028d25569943fa84ef . "./code.idejs/ide-dev.fasl")
(#x9474a53ca0b9aa32d310ecc1cd738c73 . "./code.idejs/ide-form.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
